package betterwithmods.common.blocks;

import betterwithmods.api.block.IMultiVariants;
import betterwithmods.api.block.ISoulSensitive;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.blocks.mechanical.BlockMechMachines;
import betterwithmods.util.InvUtils;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.util.*;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.Random;

import static net.minecraft.util.EnumFacing.UP;

public class BlockUrn extends BWMBlock implements ISoulSensitive, IMultiVariants {
    public static final PropertyEnum<EnumType> TYPE = PropertyEnum.func_177709_a("urntype", EnumType.class);
    public static final PropertyBool UNDERHOPPER = PropertyBool.func_177716_a("underhopper");
    private static final double OFFSET = 0.375D;
    private static final AxisAlignedBB URN_AABB = new AxisAlignedBB(0.3125D, 0, 0.3125D, 0.6875D, 0.625D, 0.6875D);
    private static final AxisAlignedBB UNDER_HOPPER_AABB = URN_AABB.func_72317_d(0, OFFSET, 0);


    public BlockUrn() {
        super(Material.field_151576_e);
        this.func_149711_c(2.0F);
        this.func_180632_j(
                this.field_176227_L.func_177621_b().func_177226_a(TYPE, EnumType.EMPTY).func_177226_a(UNDERHOPPER, false));
    }

    public static ItemStack getStack(EnumType type, int count) {
        return new ItemStack(BWMBlocks.URN, count, type.getMeta());
    }

    @Override
    public boolean func_176196_c(World worldIn, BlockPos pos) {
        BlockPos down = pos.func_177977_b();
        BlockFaceShape blockfaceshape = worldIn.func_180495_p(down).func_193401_d(worldIn, down, UP);
        boolean below = blockfaceshape != BlockFaceShape.BOWL && blockfaceshape != BlockFaceShape.UNDEFINED;

        BlockPos up = pos.func_177984_a();
        IBlockState state = worldIn.func_180495_p(up);
        boolean above = state.func_177230_c() == BWMBlocks.SINGLE_MACHINES && worldIn.func_180495_p(up).func_177229_b(BlockMechMachines.TYPE) == BlockMechMachines.EnumType.HOPPER;
        return below || above;
    }

    @Override
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos) {
        if (!func_176196_c(worldIn, pos)) {
            this.func_176226_b(worldIn, pos, state, 0);
            worldIn.func_175698_g(pos);
        }
        super.func_189540_a(state, worldIn, pos, blockIn, fromPos);
    }

    @Override
    public int func_180651_a(IBlockState state) {
        int meta = state.func_177229_b(TYPE).getMeta();
        return meta > 7 ? meta : 0;
    }

    @Override
    public String[] getVariants() {
        return new String[]{"underhopper=false,urntype=empty", "underhopper=false,urntype=12",
                "underhopper=false,urntype=25", "underhopper=false,urntype=37", "underhopper=false,urntype=50",
                "underhopper=false,urntype=62", "underhopper=false,urntype=75", "underhopper=false,urntype=87",
                "underhopper=false,urntype=full", "underhopper=false,urntype=void"};
    }

    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @Override
    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess world, BlockPos pos) {
        state = state.func_185899_b(world, pos);
        if (state.func_177229_b(UNDERHOPPER)) return UNDER_HOPPER_AABB;
        else return URN_AABB;
    }

    @Override
    public boolean isSoulSensitive(IBlockAccess world, BlockPos pos) {
        return true;
    }

    @Override
    public int getMaximumSoulIntake(IBlockAccess world, BlockPos pos) {
        int meta = world.func_180495_p(pos).func_177229_b(TYPE).getMeta();
        if (meta < 8)
            return 8 - meta;
        else if (meta == 9)
            return 64;
        return 0;
    }

    @Override
    public int processSouls(World world, BlockPos pos, int souls) {
        return Math.min(getMaximumSoulIntake(world, pos), souls);
    }

    @Override
    public boolean consumeSouls(World world, BlockPos pos, int souls) {
        int meta = world.func_180495_p(pos).func_177229_b(TYPE).getMeta();
        if (souls > 1 && meta != 9) {
            int newMeta = meta + souls;
            if (newMeta == 8) {
                InvUtils.ejectStackWithOffset(world, pos, new ItemStack(BWMBlocks.URN, 1, 8));
                return world.func_175698_g(pos);
            } else
                return world.func_175656_a(pos, func_176223_P().func_177226_a(TYPE, EnumType.byMeta(meta)));
        } else if (meta == 9)
            return true;
        return false;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void func_180655_c(IBlockState state, World world, BlockPos pos, Random rand) {
        int meta = world.func_180495_p(pos).func_177229_b(TYPE).getMeta();
        if (meta > 0) {
            int[] souls = {1, 2, 3, 4, 5, 6, 7, 8, 64};
            int chance = 80 - souls[meta - 1] / 2;
            if (rand.nextInt(chance) == 0) {
                int x = pos.func_177958_n();
                int y = pos.func_177956_o();
                int z = pos.func_177952_p();
                world.func_184134_a(pos.func_177958_n() + 0.5D, pos.func_177956_o() + 0.5D, pos.func_177952_p() + 0.5D,
                        SoundEvents.field_187551_bH, SoundCategory.BLOCKS, 1.0F, rand.nextFloat() * 0.1F + 0.45F,
                        false);
                float flX = x + rand.nextFloat();
                float flY = y + rand.nextFloat() * 0.5F + 0.625F;
                float flZ = z + rand.nextFloat();
                world.func_175688_a(EnumParticleTypes.SMOKE_NORMAL, flX, flY, flZ, 0.0D, 0.0D, 0.0D);
            }
        }
    }

    @Override
    public IBlockState func_176221_a(IBlockState state, IBlockAccess world, BlockPos pos) {
        BlockPos up = pos.func_177984_a();
        Block block = world.func_180495_p(up).func_177230_c();
        if (block != null && block == BWMBlocks.SINGLE_MACHINES) {
            if (world.func_180495_p(up).func_177229_b(BlockMechMachines.TYPE) == BlockMechMachines.EnumType.HOPPER) {
                return state.func_177226_a(UNDERHOPPER, true);
            }
        }
        return state;
    }

    @Override
    public void func_149666_a(CreativeTabs itemIn, NonNullList<ItemStack> items) {
        for (EnumType type : EnumType.VALUES)
            if (type.used)
                items.add(getStack(type, 1));
    }


    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, TYPE, UNDERHOPPER);
    }

    @Override
    public IBlockState func_176203_a(int meta) {
        return func_176223_P().func_177226_a(TYPE, EnumType.byMeta(meta));
    }

    @Override
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face) {
        return face == EnumFacing.DOWN ? BlockFaceShape.CENTER : BlockFaceShape.UNDEFINED;
    }

    @Override
    public int func_176201_c(IBlockState state) {
        return state.func_177229_b(TYPE).getMeta();
    }

    public enum EnumType implements IStringSerializable {
        EMPTY("empty", 0, true), ONE("12", 1, false), TWO("25", 2, false), THREE("37", 3, false), FOUR("50", 4, false), FIVE("62", 5, false), SIX("75",
                6, false), SEVEN("87", 7, false), FULL("full", 8, true), VOID("void", 9, true);

        private static final EnumType[] VALUES = values();
        private String name;
        private int meta;
        private boolean used;

        EnumType(String name, int meta, boolean used) {
            this.name = name;
            this.meta = meta;
            this.used = used;
        }

        public static EnumType byMeta(int meta) {
            return VALUES[meta % 10];
        }

        @Override
        public String func_176610_l() {
            return name;
        }

        public int getMeta() {
            return meta;
        }

    }
}
